@@ -3,8 +3,12 @@
 
 #[macro_use]
 pub mod error;
+#[cfg(feature = "bloom_filter")]
+pub mod bloom_filter;
 pub mod compression;
+pub mod deserialize;
 pub mod encoding;
+pub mod indexes;
 pub mod metadata;
 pub mod page;
 mod parquet_bridge;
